from pymilvus import MilvusClient, DataType, FieldSchema, CollectionSchema, Collection

# 1. 连接到 Milvus 实例
# 假设 Milvus 在本地的 19530 端口运行
client: MilvusClient = MilvusClient(uri="http://100.80.0.132:19530")

# 2. 定义 Collection 的 Schema
# Collection 类似于传统数据库中的表，用于存储具有相同结构的向量数据。
fields = [
    FieldSchema(name="id", dtype=DataType.INT64, is_primary=True, auto_id=False),
    FieldSchema(name="vector", dtype=DataType.FLOAT_VECTOR, dim=128), # 假设向量维度是 128
    FieldSchema(name="text", dtype=DataType.VARCHAR, max_length=512) # 附加的标量字段
]
schema = CollectionSchema(fields, description="This is my first Milvus collection")

# 3. 创建 Collection
collection_name = "my_test_collection"
if client.has_collection(collection_name=collection_name):
    client.drop_collection(collection_name=collection_name) # 如果已存在则删除
client.create_collection(collection_name=collection_name, schema=schema)

# 4. 准备数据并插入
# 实际应用中，这些向量会由深度学习模型生成
import random
data = []
for i in range(100):
    data.append({
        "id": i,
        "vector": [random.random() for _ in range(128)],  # 实际需要替换为 embedding 后的向量，否则查询不到数据
        "text": f"This is text number {i}"
    })

res = client.insert(collection_name=collection_name, data=data)
print(f"Inserted entities: {res}")

# 5. 创建索引
# 索引是进行高效相似性搜索的关键
index_params = client.prepare_index_params()
index_params.add_index(
    field_name="vector",
    index_type="IVF_FLAT",  # 索引类型，例如 IVF_FLAT, HNSW
    metric_type="L2",        # 距离度量，例如 L2, COSINE
    params={"nlist": 128}    # 索引参数
)
client.create_index(collection_name=collection_name, index_params=index_params)
print("Index created.")

# 6. 加载 Collection (在进行搜索前必须加载)
client.load_collection(collection_name=collection_name)
print("Collection loaded.")

# 7. 执行向量搜索
query_vector = data[0].get('vector') # 查询向量
search_results = client.search(
    collection_name=collection_name,
    data=[query_vector],
    limit=5,                  # 返回前 5 个最相似的结果
    output_fields=["text"]    # 返回结果中包含 'text' 字段
)

print(f"\nSearch Results: {search_results}")
for hit in search_results[0]:
    print(f"ID: {hit['id']}, Distance: {hit['distance']}, Text: {hit['text']}")

# 8. （可选）执行过滤搜索
filter_search_results = client.search(
    collection_name=collection_name,
    data=[query_vector],
    limit=3,
    filter="id > 50",  # 过滤条件，只搜索 id 大于 50 的实体
    output_fields=["text"]
)
print("\nFiltered Search Results (id > 50):")
for hit in filter_search_results[0]:
    print(f"ID: {hit['id']}, Distance: {hit['distance']}, Text: {hit['text']}")

# 9. （可选）删除 Collection
# client.drop_collection(collection_name=collection_name)
# print(f"Collection '{collection_name}' dropped.")
